@@ -54,6 +54,11 @@ impl MatchedPath {
         &self.absolute
     }
 
+    /// Returns the relative path.
+    pub(crate) fn relative(&self) -> &str {
+        &self.relative
+    }
+
     /// Returns the truncated absolute path.
     pub(crate) fn truncated_absolute(&self, max_width: usize) -> String {
         let chunks = self.absolute_chunks(max_width);
@@ -100,11 +105,7 @@ impl Display for MatchedPath {
 
 impl Ord for MatchedPath {
     fn cmp(&self, other: &Self) -> Ordering {
-        let depth = match self.distance().cmp(&other.distance()) {
-            Ordering::Equal => self.depth.cmp(&other.depth),
-            any => any,
-        };
-        match depth {
+        match self.distance().cmp(&other.distance()) {
             Ordering::Equal => self.relative.cmp(&other.relative),
             any => any,
         }
@@ -269,6 +270,16 @@ mod tests {
                 depth: 2,
             },
         );
+        assert_eq!(
+            new("", "/", "/abc/abc/abc.txt"),
+            MatchedPath {
+                absolute: String::from("/abc/abc/abc.txt"),
+                relative: String::from("abc/abc/abc.txt"),
+                absolute_positions: vec![],
+                relative_positions: vec![],
+                depth: 2,
+            },
+        );
         assert_eq!(
             new("abc", "/", "/abc/abc/abc.txt"),
             MatchedPath {
@@ -329,6 +340,12 @@ mod tests {
         assert_eq!(path.absolute(), "/home/abc.txt");
     }
 
+    #[test]
+    fn returns_relative() {
+        let path = new("abc", "/home", "/home/abc.txt");
+        assert_eq!(path.relative(), "abc.txt");
+    }
+
     #[test]
     fn returns_truncated_absolute() {
         let path = new("abc", "/home", "/home/☕/special/test/bar/🚞/abc.txt");
@@ -614,8 +631,8 @@ mod tests {
             given,
             vec![
                 new("abc.txt", "/home", "/home/abc.txt"),
-                new("abc.txt", "/home", "/home/src/abc.txt"),
                 new("abc.txt", "/home", "/home/abc/src/abc.txt"),
+                new("abc.txt", "/home", "/home/src/abc.txt"),
                 new("abc.txt", "/home", "/home/src/n1/n2/abc.txt"),
                 new("abc.txt", "/home", "/home/lib/abc!.txt"),
                 new("abc.txt", "/home", "/home/src/n1/n2/Foo-aXbc.txt"),
